#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "config.h"
#include "imapfilter.h"
#include "data.h"


extern account_t *accounts;
extern filter_t *filters;

unsigned int options;		/* Program options. */
char logfile[PATH_MAX];		/* Log file. */


/*
 * In the beginning there was main()...
 */
int main(int argc, char **argv)
{
    int c;
    char *confile = NULL;	/* Configuration file. */
    account_t *ca;		/* Current account. */
    mbox_t *cm;			/* Current mailbox. */

    options = OPTION_DETAILS_NORMAL;
    *logfile = 0;

    while ((c = getopt(argc, argv, "c:hl:qv")) != -1) {
	switch (c) {
	case 'c':
	    confile = optarg;
	    break;
	case 'h':
	    usage();
	    exit(ERROR_UNDEFINED);
	    break;
	case 'l':
	    strncat(logfile, optarg, PATH_MAX - 1);
	    break;
	case 'q':
	    options &= OPTION_DETAILS_CLEAR;
	    options |= OPTION_DETAILS_QUITE;
	    break;
	case 'v':
	    options &= OPTION_DETAILS_CLEAR;
	    options |= OPTION_DETAILS_VERBOSE;
	    break;
	default:
	    usage();
	    exit(ERROR_UNDEFINED);
	    break;
	}
    }

    read_config(confile);
    open_logfile();

    for (ca = accounts; ca; ca = ca->next) {

	if (init_connection(ca->server, ca->port)) {
	    close_logfile();
	    exit(ERROR_NETWORK);
	}
	server_response("\0");

#ifdef DEBUG
	test();
#endif

	login(ca->username, ca->password);

	for (cm = ca->mboxes; cm; cm = cm->next)
	    if (!*cm->filters) {
		if (!examine_mailbox(cm->name))
		    close_mailbox();
	    } else if (!select_mailbox(cm->name)) {
		apply_filters(cm->filters, cm->name);
		close_mailbox();
	    }

	logout();

	close_connection();
    }

    close_logfile();

    exit(0);
}


/*
 * Print a very brief usage message.
 */
void usage(void)
{
    fprintf(stderr,
	    "usage: imapfilter [-hqv] [-c configfile] [-l logfile]\n");
}
